/*
** element.c - Element lookup
**
** Copyright (c) 2005 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include "pjc.h"


int
changer_config_lookup(int fd,
		      int *s_transport,
		      int *n_transport,
		      int *s_storage,
		      int *n_storage,
		      int *s_ioport,
		      int *n_ioport,
		      int *s_drive,
		      int *n_drive)
{
    unsigned char buf[255];
    size_t buflen;
    int rc;
    RQS rqbuf;
    size_t rqlen = 0;
    

    rc = scsi_mode_sense_6(fd, 0, 0, 0x1D, buf, sizeof(buf), &buflen, rqbuf, &rqlen);
    if (rc != 0)
	return rc;
    
    if (buflen != 24)
	return -EINVAL;
	
    if (s_transport)
	*s_transport = ((buf[6]<<8)|buf[7]);
    
    if (n_transport)
	*n_transport = ((buf[8]<<8)|buf[9]);
    
    if (s_storage)
	*s_storage = ((buf[10]<<8)|buf[11]);
    
    if (n_storage)
	*n_storage = ((buf[12]<<8)|buf[13]);
    
    if (s_ioport)
	*s_ioport = ((buf[14]<<8)|buf[15]);
    
    if (n_ioport)
	*n_ioport = ((buf[16]<<8)|buf[17]);
    
    if (s_drive)
	*s_drive = ((buf[18]<<8)|buf[19]);

    if (n_drive)
	*n_drive = ((buf[20]<<8)|buf[21]);

    return 0;
}


int
changer_voltag_equal(const unsigned char *s1,
		     const char *s2)
{
    int s1len, s2len;

    s1len = 32;
    while (s1len > 0 && isspace(s1[s1len-1]))
	--s1len;

    s2len = strlen(s2);
    while (s2len > 0 && isspace((unsigned) s2[s2len-1]))
	--s2len;

    if (s1len != s2len)
	return 0;

    return strncmp((const char *) s1, s2, s1len) == 0;
}


int
changer_element_lookup(int fd,
		       char *obj)
{
    static unsigned char *buf = NULL;
    static size_t buflen = 0;
    static int n_transport = 0;
    static int n_storage = 0;
    static int n_ioport = 0;
    static int n_drive = 0;
    static int n_elems = 0;
    char *cp;
    int n, i, j, elemtypes, rc, p, id, addr, elem_t, full;
    RQS rqbuf;
    size_t rqlen = 0;
    

    if (sscanf(obj, "#%u", &id) == 1 ||
	sscanf(obj, "%u", &id) == 1)
	return id;

    cp = strchr(obj, ':');
    if (!cp)
    {
	elem_t = 0; /* Lookup by label ID */
    }
    else
    {
	int len;

	
	++cp;
	len = cp-obj;
	
	if (strncmp(obj, "transport:", len) == 0)
	    elem_t = 1;
	else if (strncmp(obj, "storage:", len) == 0)
	    elem_t = 2;
	else if (strncmp(obj, "ioport:", len) == 0)
	    elem_t = 3;
	else if (strncmp(obj, "drive:", len) == 0)
	    elem_t = 4;
	else
	    return -1;
	
	if (sscanf(cp, "%u", &id) != 1)
	    return -1;
    }


    if (buf == NULL)
    {
	n_transport = n_storage = n_ioport = n_drive = 0;
	rc = changer_config_lookup(fd,
				   NULL, &n_transport, NULL, &n_storage,
				   NULL, &n_ioport, NULL, &n_drive);
	if (rc != 0)
	    return -1;
	
	if (n_transport < 1)
	    n_transport = 1;
	
	n_elems = n_transport+n_storage+n_ioport+n_drive;
	buflen = 1024+n_elems*256;
	buf = malloc(buflen);
	if (!buf)
	    return -1;
	
	rc = scsi_read_element_status(fd, 0, 0, n_elems, buf, buflen, &buflen, rqbuf, &rqlen);
	if (rc != 0)
	    goto Fail;
    }
	

    p = 0;
    elemtypes = (buf[p+2]<<8) + buf[p+3];
    
    p += 8;
    for (i = 0; i < elemtypes;)
    {
	size_t elength, dlength;
	int etype, pvoltag, avoltag;
	
	etype = buf[p];
	pvoltag = (buf[p+1]&0x80);
	avoltag = (buf[p+1]&0x40);
	
	elength = (buf[p+2]<<8) + buf[p+3];
	dlength = (buf[p+5]<<16) + (buf[p+6]<<8) + buf[p+7];
	
	p += 8;

	n = element_start;
	
	for (j = 0; j < dlength; j += elength, p += elength, n++)
	{
	    int xp;
	    
	    ++i;

	    addr = (buf[p+0]<<8) + buf[p+1];
	    full = (buf[p+2] & 0x01);
	    
	    if (elem_t > 0 && elem_t == etype && n == id)
		goto Done;

	    if (!elem_t)
		switch (etype)
		{
		  case 1: /* Medium Transport Element */
		    xp = p+12;
		    if (pvoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
			    
			xp += 36;
		    }
		    if (avoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
		    }
		    break;
		    
		  case 2: /* Storage Element */
		    xp = p+12;
		    if (pvoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
			xp += 36;
		    }
		    if (avoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
		    }
		    break;

		  case 3: /* Import/Export Element */
		    xp = p+12;
		    if (pvoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
			xp += 36;
		    }
		    if (avoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
		    }
		    break;
		    
		  case 4: /* Data Transfer Element */
		    xp = p+12;
		    if (pvoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
			xp += 36;
		    }
		    if (avoltag)
		    {
			if (changer_voltag_equal(buf+xp, obj))
			    goto Done;
		    }
		    break;
		}
	}
    }

#if 0
    free(buf);
#endif
    return 0;
    
  Fail:
#if 0
    free(buf);
#endif
    return -1;
    
  Done:
#if 0
    free(buf);
#endif
    return addr;

}

